#include "cellulamainwindow.h"
#include <QTime>
#include <QMenuBar>
#include <QMenu>
#include <QMdiSubWindow>

CellulaMainWindow::CellulaMainWindow()
{
    createAction();
    createMenu();
    createToolbar();
    translateGUI();

    option_dialog = new Options(this);
    setWindowIcon(QIcon(":/images/glide.svg"));

    mdi = new QMdiArea(this);

    //gl_widget = new GLwidget(QGLFormat(QGL::DoubleBuffer), this);
    //gl_widget->resize(0, 0);

    setCentralWidget(mdi);
    setGeometry(0,0, 800, 600);
    connect(actionNew, SIGNAL(activated()), this, SLOT(showOptionDialog()));
    connect(actionOpen, SIGNAL(activated()), this, SLOT(newModel()));
    connect(actionRun, SIGNAL(activated()), this, SLOT(startSimulation()));
    connect(actionPause, SIGNAL(activated()), this, SLOT(stopSimulation()));
    connect(actionQuit, SIGNAL(activated()), this, SLOT(close()));
    connect(actionTile, SIGNAL(activated()), mdi, SLOT(tileSubWindows()));
    connect(actionCascade, SIGNAL(activated()), mdi, SLOT(cascadeSubWindows()));

    machine = 0;
    createCLMachine();
}

CellulaMainWindow::~CellulaMainWindow()
{
    //delete model;
    delete machine;
}

void CellulaMainWindow::createMenu()
{
    QMenuBar *menuBar = new QMenuBar(this);
    QMenu *file_menu = new QMenu(trUtf8("Súbor"), menuBar);
    file_menu->addAction(actionNew);
    file_menu->addAction(actionOpen);
    file_menu->addAction(actionQuit);

    QMenu *run_menu = new QMenu(trUtf8("Beh"), menuBar);
    run_menu->addAction(actionRun);
    run_menu->addAction(actionPause);

    QMenu *window_menu = new QMenu(trUtf8("Okná"), menuBar);
    window_menu->addAction(actionTile);
    window_menu->addAction(actionCascade);

    menuBar->addMenu(file_menu);
    menuBar->addMenu(run_menu);
    menuBar->addMenu(window_menu);
    setMenuBar(menuBar);
}

void CellulaMainWindow::createToolbar()
{
    main_toolbar = new QToolBar(trUtf8("Hlavný panel nástrojov"), this);
    main_toolbar->addAction(actionNew);
    main_toolbar->addAction(actionOpen);
    main_toolbar->addAction(actionRun);
    main_toolbar->addAction(actionPause);
    addToolBar(main_toolbar);
}

void CellulaMainWindow::translateGUI()
{
    setWindowTitle(trUtf8("Cellula"));
}

void CellulaMainWindow::createAction()
{
    //QCleanlooksStyle cur_style;
    //QIcon icon = cur_style.standardIcon(QStyle::SP_DirHomeIcon);
    actionNew = new QAction(trUtf8("Nový model"), this);
    actionOpen = new QAction(trUtf8("Otvor model"), this);
    actionRun = new QAction(trUtf8("Spusti simuláciu"), this);
    actionPause = new QAction(trUtf8("Pozastav simuláciu"), this);
    actionQuit = new QAction(trUtf8("Koniec"), this);
    actionTile = new QAction(trUtf8("Dlaždice"), this);
    actionCascade = new QAction(trUtf8("Kaskáda"), this);
}

void CellulaMainWindow::showOptionDialog()
{
    option_dialog->show();
}

void CellulaMainWindow::createCLMachine()
{
    //delete machine;
    //gl_widget->makeCurrent();
    //gl_widget->updateGL();
    //machine = new Machine((cl_context_properties)glXGetCurrentContext(), (cl_context_properties)glXGetCurrentDisplay(), option_dialog->getPlatform(), option_dialog->getDevices());
    machine = new Machine(option_dialog->getPlatform(), option_dialog->getDevices());
    model = new SynchronousModel(machine, "");

    /*unsigned char *tex = space->renderToTexture();
    gl_widget->loadTexture(space->getWidth(), space->getHeight(), tex);
    delete [] tex;*/
    asyn_model = new AsynchronousModel(machine, "");
    GLwidget *asyn_widget = new GLwidget(QGLFormat(QGL::DoubleBuffer), this, new CellSpace(asyn_model, 16, 16));
    float asyn_data[] = {1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
    ((AsynchronousModel*)asyn_model)->initialize(asyn_widget->getSpace(), asyn_data);
    QMdiSubWindow *subwindow = mdi->addSubWindow(asyn_widget);
    subwindow->show();
    subwindow->setWindowTitle(trUtf8("Asynchronny model"));
}

void CellulaMainWindow::startSimulation()
{
    GLwidget *active_window = dynamic_cast<GLwidget*>(mdi->activeSubWindow() ? mdi->activeSubWindow()->widget() : 0);
    if(active_window)
        active_window->startSimulation();
}

void CellulaMainWindow::stopSimulation()
{
    GLwidget *active_window = dynamic_cast<GLwidget*>(mdi->activeSubWindow() ? mdi->activeSubWindow()->widget() : 0);
    if(active_window)
        active_window->stopSimulation();
}

void CellulaMainWindow::newModel()
{
    GLwidget *new_glwidget = new GLwidget(QGLFormat(QGL::DoubleBuffer), this, new CellSpace(model, 256, 256));
    QMdiSubWindow *subwindow = mdi->addSubWindow(new_glwidget);
    subwindow->show();
    subwindow->setWindowTitle(trUtf8("Synchronny model"));
}
